% sines.m - generate sum of real or complex decaying sinusoids in noise
%
% Usage: [y,s] = sines(N,w,SNR,phi,seed,type)
%        [y,s] = sines(N,w,SNR,phi,seed)      (equivalent to type=0, real sinusoids)
%
% N    = desired length of signal y(n)
% w    = length-r vector of complex (w=om+ja) sinusoid frequencies (in rads/sample) 
% SNR  = length-r vector of SNR's (in dB)
% phi  = length-r vector of phases (in radians) over [0,2*pi]
% seed = initialization seed for random number generator
% type = 0,1 for real or complex sinusoids (default type=0, real case)
%
% y = length-N signal (y = s+v, where v=noise)
% s = length-N noiseless signal 
%
% notes: complex frequencies w = om+ja, om = frequencies, a = damping factors,
%        SNRs are in dB, random phi's can be generated by phi = 2*pi*rand(r,1),
%        w,SNR,phi can be entered as rows or columns
%
%        type=0: y(n) = \sum_{i=1}^r A(i) * e^{-a(i)*n)} * cos(w(i)*n + phi(i)) + v(n),
%                where v(n) is zero-mean real-valued white noise of variance sigmav^2 = 1
%                signal/noise ratio SNR(i) = P(i)/N = (A(i)^2/2)/sigmav^2 = A(i)^2/2
%
%        type=1: y(n) = \sum_{i=1}^r A(i) * e^{-a(i)*n} * e^{j*w(i)*n + j*phi(i)} + v(n),
%                v(n) is zero-mean complex-valued white noise of variance sigmav^2 = 2
%                signal/noise ratio SNR(i) = P(i)/N = A(i)^2/sigmav^2 = A(i)^2/2
%        
%        for both types, the SNRs in dB are SNR(i)=10*log10(A(i)^2/2), i=1,2,...,r

% S. J. Orfanidis - 1999
% ECE Department
% Rutgers University
% Piscataway, NJ 08854
% email: orfanidi@ece.rutgers.edu

function [y,s] = sines(N,w,SNR,phi,seed,type)

if nargin==0, help sines; return; end
if nargin==5, type=0; end

r = length(w);                                          % number of sinusoids

z = exp(j*w);                                           % sinusoid zeros z=exp(-a+j*om)

A = sqrt(2) * 10.^(SNR(:)/20) .* exp(j*phi(:));         % complex sinusoid amplitudes

s = steermat(N-1,z) * A;                                % sinusoidal part

randn('state', seed);                                   % seed initialization

v = randn(N,1) + j*randn(N,1);                          % complex noise of variance 2

if type==0,
    s = real(s);                                                          
    v = real(v);                                        % v = real(v) has variance 1
end

y = s + v;






